@@ -4,24 +4,22 @@ from math import ceil, floor
 
 from affine import Affine
 import numpy as np
+
 import rasterio
 
 from rasterio._base import _transform
+from rasterio.crs import CRS
 from rasterio.enums import Resampling
-from rasterio.env import GDALVersion, ensure_env, require_gdal_version
-from rasterio.errors import GDALBehaviorChangeException, TransformError
+from rasterio.env import ensure_env, require_gdal_version
+from rasterio.errors import TransformError, RPCError
 from rasterio.transform import array_bounds
-from rasterio._warp import _calculate_default_transform, _reproject, _transform_geom
-
-# Gauss (7) is not supported for warp
-SUPPORTED_RESAMPLING = [r for r in Resampling if r.value < 7]
-GDAL2_RESAMPLING = [r for r in Resampling if r.value > 7 and r.value <= 12]
-if GDALVersion.runtime().at_least('2.0'):
-    SUPPORTED_RESAMPLING.extend(GDAL2_RESAMPLING)
-# rms supported since GDAL 3.3
-if GDALVersion.runtime().at_least('3.3'):
-    SUPPORTED_RESAMPLING.append(Resampling.rms)
-
+from rasterio._warp import (
+    _calculate_default_transform,
+    _reproject,
+    _transform_bounds,
+    _transform_geom,
+    SUPPORTED_RESAMPLING,
+)
 
 @ensure_env
 def transform(src_crs, dst_crs, xs, ys, zs=None):
@@ -125,7 +123,7 @@ def transform_bounds(
     Optionally densifying the edges (to account for nonlinear transformations
     along these edges) and extracting the outermost bounds.
 
-    Note: this does not account for the antimeridian.
+    Note: antimeridian support added in version 1.3.0
 
     Parameters
     ----------
@@ -146,40 +144,20 @@ def transform_bounds(
     left, bottom, right, top: float
         Outermost coordinates in target coordinate reference system.
     """
-    if densify_pts < 0:
-        raise ValueError('densify parameter must be >= 0')
-
-    in_xs = []
-    in_ys = []
-
-    if densify_pts > 0:
-        densify_factor = 1.0 / float(densify_pts + 1)
-
-        # Add points along outer edges.
-        for x in (left, right):
-            in_xs.extend([x] * (densify_pts + 2))
-            in_ys.extend(
-                bottom + np.arange(0, densify_pts + 2, dtype=np.float64) *
-                ((top - bottom) * densify_factor)
-            )
-
-        for y in (bottom, top):
-            in_xs.extend(
-                left + np.arange(1, densify_pts + 1, dtype=np.float64) *
-                ((right - left) * densify_factor)
-            )
-            in_ys.extend([y] * densify_pts)
-
-    else:
-        in_xs = [left, left, right, right]
-        in_ys = [bottom, top, bottom, top]
-
-    xs, ys = transform(src_crs, dst_crs, in_xs, in_ys)
-    return (min(xs), min(ys), max(xs), max(ys))
+    src_crs = CRS.from_user_input(src_crs)
+    dst_crs = CRS.from_user_input(dst_crs)
+    return _transform_bounds(
+        src_crs,
+        dst_crs,
+        left,
+        bottom,
+        right,
+        top,
+        densify_pts,
+    )
 
 
 @ensure_env
-@require_gdal_version('2.0', param='resampling', values=GDAL2_RESAMPLING)
 def reproject(source, destination=None, src_transform=None, gcps=None, rpcs=None,
               src_crs=None, src_nodata=None, dst_transform=None, dst_crs=None,
               dst_nodata=None, dst_resolution=None, src_alpha=0, dst_alpha=0,
@@ -251,20 +229,9 @@ def reproject(source, destination=None, src_transform=None, gcps=None, rpcs=None
         Index of a band to use as the alpha band when warping.
     dst_alpha : int, optional
         Index of a band to use as the alpha band when warping.
-    resampling: int
-        Resampling method to use.  One of the following:
-            Resampling.nearest,
-            Resampling.bilinear,
-            Resampling.cubic,
-            Resampling.cubic_spline,
-            Resampling.lanczos,
-            Resampling.average,
-            Resampling.mode,
-            Resampling.max (GDAL >= 2.2),
-            Resampling.min (GDAL >= 2.2),
-            Resampling.med (GDAL >= 2.2),
-            Resampling.q1 (GDAL >= 2.2),
-            Resampling.q3 (GDAL >= 2.2)
+    resampling: int, rasterio.enums.Resampling
+        Resampling method to use.
+        Default is :attr:`rasterio.enums.Resampling.nearest`.
         An exception will be raised for a method not supported by the running
         version of GDAL.
     num_threads : int, optional
@@ -279,7 +246,10 @@ def reproject(source, destination=None, src_transform=None, gcps=None, rpcs=None
         raster to a destination of the same size is approximately
         56 MB. The default (0) means 64 MB with GDAL 2.2.
     kwargs:  dict, optional
-        Additional arguments passed to transformation function.
+        Additional arguments passed to both the image to image
+        transformer :cpp:func:`GDALCreateGenImgProjTransformer2` (for example,
+        MAX_GCP_ORDER=2) and the :cpp:struct:`GDALWarpOptions` (for example,
+        INIT_DEST=NO_DATA).
 
     Returns
     ---------
@@ -319,7 +289,7 @@ def reproject(source, destination=None, src_transform=None, gcps=None, rpcs=None
             else:
                 src_count = 1
                 src_height, src_width = source.shape
-            
+
             # try to compute src_bounds if we don't have gcps
             if not (gcps or rpcs):
                 src_bounds = array_bounds(src_height, src_width, src_transform)
@@ -331,12 +301,23 @@ def reproject(source, destination=None, src_transform=None, gcps=None, rpcs=None
             if not (src_rdr.transform.is_identity and src_rdr.crs is None):
                 src_bounds = src_rdr.bounds
 
-            src_crs = src_rdr.crs
+            src_crs = src_crs or src_rdr.crs
+
+            # raise exception when reprojecting with rpcs using a CRS that is not EPSG:4326
+            if rpcs:
+                if isinstance(src_crs, str):
+                    src_crs_obj = rasterio.crs.CRS.from_string(src_crs)
+                else:
+                    src_crs_obj = src_crs
+                if src_crs is not None and src_crs_obj.to_epsg() != 4326:
+                    raise RPCError("Reprojecting with rational polynomial coefficients using source CRS other than EPSG:4326")
+
             if isinstance(src_bidx, int):
                 src_bidx = [src_bidx]
+
             src_count = len(src_bidx)
             src_height, src_width = src_shape
-            gcps = src_rdr.gcps[0] if src_rdr.gcps[0] else None
+            gcps = src_rdr.gcps[0] if src_rdr.gcps[0] and not rpcs else None
 
         dst_height = None
         dst_width = None
@@ -469,9 +450,11 @@ def calculate_default_transform(
     Some behavior of this function is determined by the
     CHECK_WITH_INVERT_PROJ environment variable:
 
-        YES: constrain output raster to extents that can be inverted
-             avoids visual artifacts and coordinate discontinuties.
-        NO:  reproject coordinates beyond valid bound limits
+        YES
+            constrain output raster to extents that can be inverted
+            avoids visual artifacts and coordinate discontinuties.
+        NO
+            reproject coordinates beyond valid bound limits
     """
     if any(x is not None for x in (left, bottom, right, top)) and gcps:
         raise ValueError("Bounding values and ground control points may not"
@@ -483,7 +466,7 @@ def calculate_default_transform(
     if any(x is None for x in (left, bottom, right, top)) and not (gcps or rpcs):
         raise ValueError("Either four bounding values, ground control points,"
                          " or rational polynomial coefficients must be specified")
-    
+
     if gcps and rpcs:
         raise ValueError("ground control points and rational polynomial",
                          " coefficients may not be used together.")
@@ -528,14 +511,14 @@ def calculate_default_transform(
 
         dst_width = ceil(dst_width * xratio)
         dst_height = ceil(dst_height * yratio)
-    
+
     if dimensions:
         xratio = dst_width / dimensions[0]
         yratio = dst_height / dimensions[1]
 
         dst_width = dimensions[0]
         dst_height = dimensions[1]
-        
+
         dst_affine = Affine(dst_affine.a * xratio, dst_affine.b, dst_affine.c,
                             dst_affine.d, dst_affine.e * yratio, dst_affine.f)
 
