@@ -27,10 +27,8 @@ from affine import Affine
 import attr
 import numpy as np
 
-import rasterio._loading
-with rasterio._loading.add_gdal_dll_directories():
-    from rasterio.errors import WindowError, RasterioDeprecationWarning
-    from rasterio.transform import rowcol, guard_transform
+from rasterio.errors import WindowError, RasterioDeprecationWarning
+from rasterio.transform import rowcol, guard_transform
 
 
 class WindowMethodsMixin:
@@ -162,7 +160,10 @@ def get_data_window(arr, nodata=None):
     # Otherwise retrieve mask from array (if it is masked)
     # Finally try returning a full window (nodata=None and nothing in arr is masked)
     if nodata is not None:
-        arr_mask = arr != nodata
+        if np.isnan(nodata):
+            arr_mask = ~np.isnan(arr)
+        else:
+            arr_mask = arr != nodata
     elif np.ma.is_masked(arr):
         arr_mask = ~np.ma.getmask(arr)
     else:
@@ -182,10 +183,10 @@ def get_data_window(arr, nodata=None):
             v.append((nz.min(), nz.max() + 1))
         else:
             v.append((0, 0))
-    
+
     if arr_mask.ndim == 1:
         v.append((0, 0))
-        
+
     return Window.from_slices(*v)
 
 
@@ -708,8 +709,8 @@ class Window:
     def round_lengths(self, **kwds):
         """Return a copy with width and height rounded.
 
-        Lengths are rounded to the preceding (floor) or succeeding (ceil)
-        whole number. The offsets are not changed.
+        Lengths are rounded to the nearest whole number. The offsets are
+        not changed.
 
         Parameters
         ----------
@@ -736,8 +737,8 @@ class Window:
     def round_offsets(self, **kwds):
         """Return a copy with column and row offsets rounded.
 
-        Offsets are rounded to the preceding (floor) or succeeding (ceil)
-        whole number. The lengths are not changed.
+        Offsets are rounded to the preceding whole number. The lengths
+        are not changed.
 
         Parameters
         ----------
